package game;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.SwingUtilities;
import javax.swing.Timer;

import sew.SewLib;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.awt.Color;
import java.awt.Font;


/****************************************************************************************************
 * Das gesamte Spielfeld, erstellen, loeschen von Pipes, Timer, usw.
 * 
 * @author Daniel Hamm
 * @version 1.0
 *
 ****************************************************************************************************
 */

//---------------------------------------------------------------------------------------------------
public class Spielfeld extends JPanel implements _ISpielfeld
{
	private static final int ENTFERNUNG   = 80;
	private static final int SIZE         = 70;

	/**
	 * 2d Array aus Pipes - das Spielfeld
	 */
	private Pipe[][]         field        = new Pipe[4][6];

	/**
	 * Auf welchem Level ist der Spieler
	 */
	private int              level        = 1;
	/**
	 * Wie viele Punkte hat der Spieler
	 */
	private int              points       = 0;
	/**
	 * Mit welcher Schwierigkeit spielt der Spieler
	 */
	private int              difficulty   = 0;
	String                   timerString;
	/**
	 * Wie viel Zeit uebrig ist
	 */
	int                      timeLeft;

	private JLabel           lbl_Timer;

	Timer                    timer;

	/**
	 * String Array, fuer die Aufstellung der Level
	 */
	private String[]         levelsString = getLevels();

	/**
	 * String Array, fuer die Loesungen der Level
	 */
	private String[]         solution     = getSolution();

	public String[] getLevels()
	{
		String[] levels = new String[20];
		int      index  = 0;

		try
		{
			FileReader     fileReader     = new FileReader(new File("src//game//Levels.txt"));
			BufferedReader bufferedReader = new BufferedReader(fileReader);
			String         line;

			while ((line = bufferedReader.readLine()) != null)
			{
				levels[index] = line;
				index++;
			}

		} catch (IOException e)
		{
			e.printStackTrace();
		}
		return levels;
	}

	public void won()
	{
		level++;

		if (difficulty == 0)
			points += 250;
		else if (difficulty == 1)
			points += 500;
		else
			points += 1000;

		createPipes();
	}

	public void start()
	{
		if (difficulty == 0)
			timeLeft = 25;
		else if (difficulty == 1)
			timeLeft = 20;
		else
			timeLeft = 15;

		timer = new Timer(1000, new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent aE)
			{
				timeLeft--;
				timerString = new String("Zeit verbleibend : " + timeLeft + "s");
				lbl_Timer.setText(timerString);
				if (timeLeft == 0)
				{
					waterFlows();
					timer.stop();
				}
			}
		});
		timer.start();
	}

	public String[] getSolution()
	{
		String[] solution = new String[20];
		int      index    = 0;

		try
		{
			FileReader     fileReader     = new FileReader(new File("src//game//Solution.txt"));
			BufferedReader bufferedReader = new BufferedReader(fileReader);
			String         line;

			while ((line = bufferedReader.readLine()) != null)
			{
				solution[index] = line;
				index++;
			}

		} catch (IOException e)
		{
			e.printStackTrace();
		}
		return solution;
	}

	public String getPath()
	{
		String path = "";

		for (Pipe[] pipes : field)
		{
			for (Pipe pipe : pipes)
			{
				if (pipe instanceof Pipe_Corner)
					path += pipe.getDirection();
				else if (pipe instanceof Pipe_Cross)
					path += "c";
				else if (pipe instanceof Pipe_Straight)
				{
					if (pipe.getDirection() % 2 == 0)
						path += "i";
					else if (pipe.getDirection() % 2 == 1)
						path += "-";
				}
				else
					path += "o";
			}
		}

		return path;
	}

	public void waterFlows()
	{
		char[] currentPath = getPath().toCharArray();

		for (int index = 0; index <= solution[0].length() - 1; index++)
		{
			if (solution[level - 1].charAt(index) == 'x')
				currentPath[index] = 'x';
		}

		String path = new String(currentPath);

		if (path.equals(solution[level - 1]))
			won();
		else
			lost();
	}

	public void lost()
	{
		String lostMSG   = "Sie haben verloren!\nEndpunkte: " + points;
		String lostTitle = "Spiel verloren";
		JOptionPane.showMessageDialog(null, lostMSG, lostTitle, JOptionPane.OK_OPTION);

		System.exit(0);
	}

	public void removePipes()
	{
		for (Pipe[] pipes : field)
		{
			for (Pipe pipe : pipes)
			{
				if (pipe != null)
					remove(pipe);
			}
		}
		field = new Pipe[4][6];

		repaint();
	}

	public void createPipes()
	{
		removePipes();

		String pipes = levelsString[level - 1];

		int    index = 0;

		for (int spalte = 0; spalte < 4; spalte++)
		{
			for (int reihe = 0; reihe < 6; reihe++)
			{
				if (pipes.charAt(index) == 'x')
				{
					JLabel btn_Pipe = new Pipe_Cross();
					btn_Pipe.addMouseListener(new MouseListener()
					{
						@Override
						public void mouseClicked(MouseEvent e)
						{
							if (SwingUtilities.isLeftMouseButton(e))
							{
								((Pipe) btn_Pipe).leftRotate();
								System.out.printf("Text \n");
							}
							else if (SwingUtilities.isRightMouseButton(e))
								((Pipe) btn_Pipe).rightRotate();
						}

						@Override
						public void mousePressed(MouseEvent aE)
						{
						}

						@Override
						public void mouseReleased(MouseEvent aE)
						{
						}

						@Override
						public void mouseEntered(MouseEvent aE)
						{
						}

						@Override
						public void mouseExited(MouseEvent aE)
						{
						}
					});
					btn_Pipe.setBounds(ENTFERNUNG + spalte * 75, ENTFERNUNG + reihe * 75, SIZE, SIZE);
					add(btn_Pipe);
					field[spalte][reihe] = (Pipe) btn_Pipe;
					((Pipe) btn_Pipe).setDirection(SewLib.random(0, 3));
					((Pipe_Cross) btn_Pipe).paint(((Pipe) btn_Pipe).getDirection());
				}
				else if (pipes.charAt(index) == 'v')
				{
					JLabel btn_Pipe = new Pipe_Corner();
					btn_Pipe.addMouseListener(new MouseListener()
					{
						@Override
						public void mouseClicked(MouseEvent e)
						{
							if (SwingUtilities.isLeftMouseButton(e))
								((Pipe) btn_Pipe).leftRotate();
							else if (SwingUtilities.isRightMouseButton(e))
								((Pipe) btn_Pipe).rightRotate();
						}

						@Override
						public void mousePressed(MouseEvent aE)
						{
						}

						@Override
						public void mouseReleased(MouseEvent aE)
						{
						}

						@Override
						public void mouseEntered(MouseEvent aE)
						{
						}

						@Override
						public void mouseExited(MouseEvent aE)
						{
						}
					});
					btn_Pipe.setBounds(ENTFERNUNG + spalte * 75, ENTFERNUNG + reihe * 75, SIZE, SIZE);
					add(btn_Pipe);
					field[spalte][reihe] = (Pipe) btn_Pipe;
					((Pipe) btn_Pipe).setDirection(SewLib.random(0, 3));
					((Pipe_Corner) btn_Pipe).paint(((Pipe) btn_Pipe).getDirection());
				}
				else if (pipes.charAt(index) == 'i')
				{
					JLabel btn_Pipe = new Pipe_Straight();
					btn_Pipe.addMouseListener(new MouseListener()
					{
						@Override
						public void mouseClicked(MouseEvent e)
						{
							if (SwingUtilities.isLeftMouseButton(e))
								((Pipe) btn_Pipe).leftRotate();
							else if (SwingUtilities.isRightMouseButton(e))
								((Pipe) btn_Pipe).rightRotate();
						}

						@Override
						public void mousePressed(MouseEvent aE)
						{
						}

						@Override
						public void mouseReleased(MouseEvent aE)
						{
						}

						@Override
						public void mouseEntered(MouseEvent aE)
						{
						}

						@Override
						public void mouseExited(MouseEvent aE)
						{
						}
					});
					btn_Pipe.setBounds(ENTFERNUNG + spalte * 75, ENTFERNUNG + reihe * 75, SIZE, SIZE);
					add(btn_Pipe);
					field[spalte][reihe] = (Pipe) btn_Pipe;
					((Pipe) btn_Pipe).setDirection(SewLib.random(0, 3));
					((Pipe_Straight) btn_Pipe).paint(((Pipe) btn_Pipe).getDirection());
				}
				index++;
			}
		}

		if (level == 10)
			JOptionPane.showConfirmDialog(null,
					"X-Rohre leiten nicht in alle Richtungen, sie sind 2 überlappende I-Rohre", "Tipp",
					JOptionPane.OK_OPTION);

		start();
	}

	/**
	 * @return Auf welchem Level der Spieler ist
	 */
	public int getLevel()
	{
		return level;
	}

	/**
	 * @return Anzahl der Punkte
	 */
	public int getPoints()
	{
		return points;
	}

	/**
	 * Create the panel.
	 */
	public Spielfeld(int difficulty)
	{
		this.difficulty = difficulty;

		setBackground(new Color(102, 51, 0));
		setLayout(null);

		createPipes();

		JLabel lbl_Start = new JLabel("Das Wasser \r\nkommt von hier");
		lbl_Start.setForeground(new Color(255, 255, 0));
		lbl_Start.setFont(new Font("Tahoma", Font.PLAIN, 15));
		lbl_Start.setBackground(new Color(0, 204, 255));
		lbl_Start.setBounds(0, 49, 235, 19);
		add(lbl_Start);

		JLabel lbl_Finish = new JLabel("Das Wasser muss hier her");
		lbl_Finish.setForeground(new Color(255, 255, 0));
		lbl_Finish.setFont(new Font("Tahoma", Font.PLAIN, 15));
		lbl_Finish.setBackground(new Color(0, 204, 255));
		lbl_Finish.setBounds(377, 499, 235, 19);
		add(lbl_Finish);

		lbl_Timer = new JLabel("Zeit verbleiben: 18s");
		lbl_Timer.setFont(new Font("Tahoma", Font.PLAIN, 15));
		lbl_Timer.setForeground(Color.YELLOW);
		lbl_Timer.setBounds(387, 49, 172, 19);
		add(lbl_Timer);
	}
}
