package plf1;
import static org.junit.jupiter.api.Assertions.*;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Order;

/***************************************************************************************************
 * 
 * JUnit Test für Haustiere
 *
 * @author Prof. Gaisberger
 * @version 1.0
 * 
 ***************************************************************************************************
 */

@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
//---------------------------------------------------------------------------------------------------
class _JUT_Haustiere
{
	private static final String errNewNoError = "NEW: Mit diesen Parametern darf keine Exception ausgelöst werden!";
	private static final String errNewNullPointer  = "NEW: Parameter darf nicht NULL sein (EXCEPTION fehlt): ";;
	private static final String errNewEmptyStr = "NEW: Parameter darf kein Leerstring sein (EXCEPTION fehlt): ";
	private static final String errArgIllegal = "Ungültiger Wert! (EXCEPTION fehlt): ";
	private static final String errArgNoError = "Dieser Wert ist gültig! Hier darf KEINE Exception ausgelöst werden: ";
	private static final String errReturnValue = "Der zurückgegeben Wert ist ungültig! Methode: ";
//	private static final String errReturnListe = "Die zurückgegebene Liste ist noch nicht korrekt! Methode: ";

	// -----------------------------------------------------------------
	@Test
	@Order(10)
	void testKatze()
	{
		// --------------------------------------------------------------------------------
		// Test new mit Null und Leerstrings!!!!
		assertThrows(NullPointerException.class, () -> {
			new Katze(null, "Katze", "Weiss", 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Katze("", "Katze", "Weiss", 99);
		}, errNewEmptyStr);

		assertThrows(NullPointerException.class, () -> {
			new Katze("Minki", null, "Weiss", 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Katze("Minki", "", "Weiss", 99);
		}, errNewEmptyStr);

		assertThrows(NullPointerException.class, () -> {
			new Katze("Minki", "Katze", null, 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Katze("Minki", "Katze", "", 99);
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Katze("Minki", "Katze", "Weiss", -1);
		}, errArgIllegal + "Alter");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Katze("Minki", "Katze", "Weiss", 99);
		}, errNewNoError);
		
		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Katze neu = new Katze("Minki", "Katze", "Weiss", 99);
		
		assertEquals("Minki",  neu.getName(),   errReturnValue + "getName");
		assertEquals(99,       neu.getAlter(),  errReturnValue + "getAlter");
		assertEquals("Weiss",  neu.getFarbe(),  errReturnValue + "getFarbe");
		assertEquals("Katze",  neu.getRasse(),  errReturnValue + "getRasse");

		
		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			neu.setName("Test");
		}, errArgNoError);
		assertEquals("Test", neu.getName(), errReturnValue + "getName");

		assertThrows(NullPointerException.class, () -> {
			neu.setName(null);
		}, errArgIllegal + "setName");
		assertThrows(IllegalArgumentException.class, () -> {
			neu.setName("");
		}, errArgIllegal + "setName");

		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			neu.setAlter(3);
		}, errArgNoError);
		assertEquals(3, neu.getAlter(), errReturnValue + "getAlter");

		assertThrows(IllegalArgumentException.class, () -> {
			neu.setAlter(-1);
		}, errArgIllegal + "Alter");

		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			neu.setLieblingsFutter("Maus");
		}, errArgNoError);
		assertEquals("Maus", neu.getLieblingsFutter(), errReturnValue + "getLieblingsFutter");

		assertThrows(NullPointerException.class, () -> {
			neu.setLieblingsFutter(null);
		}, errArgIllegal + "setLieblingsFutter");

		
		// --------------------------------------------------------------------------------
		neu.bewegeDich();
		neu.frissEtwas();
		neu.gibLaut();
		neu.streicheln();
	}

	@Test
	@Order(11)
	void testHase()
	{
		// --------------------------------------------------------------------------------
		// Test new mit Null und Leerstrings!!!!
		assertThrows(NullPointerException.class, () -> {
			new Hase(null, "Hase", "Weiss", 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Hase("", "Hase", "Weiss", 99);
		}, errNewEmptyStr);

		assertThrows(NullPointerException.class, () -> {
			new Hase("Lotte", null, "Weiss", 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Hase("Lotte", "", "Weiss", 99);
		}, errNewEmptyStr);

		assertThrows(NullPointerException.class, () -> {
			new Hase("Lotte", "Hase", null, 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Hase("Lotte", "Hase", "", 99);
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Hase("Lotte", "Hase", "Weiss", -1);
		}, errArgIllegal + "Alter");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Hase("Lotte", "Hase", "Weiss", 99);
		}, errNewNoError);

		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Hase neu = new Hase("Lotte", "Hase", "Weiss", 99);
		
		assertEquals("Lotte",  neu.getName(),   errReturnValue + "getName");
		assertEquals(99,       neu.getAlter(),  errReturnValue + "getAlter");
		assertEquals("Weiss",  neu.getFarbe(),  errReturnValue + "getFarbe");
		assertEquals("Hase",   neu.getRasse(),  errReturnValue + "getRasse");

		
		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			neu.setName("Test");
		}, errArgNoError);
		assertEquals("Test", neu.getName(), errReturnValue + "getName");

		assertThrows(NullPointerException.class, () -> {
			neu.setName(null);
		}, errArgIllegal + "setName");
		assertThrows(IllegalArgumentException.class, () -> {
			neu.setName("");
		}, errArgIllegal + "setName");


		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			neu.setAlter(3);
		}, errArgNoError);
		assertEquals(3, neu.getAlter(), errReturnValue + "getAlter");

		assertThrows(IllegalArgumentException.class, () -> {
			neu.setAlter(-1);
		}, errArgIllegal + "setAlter");

		// --------------------------------------------------------------------------------
		neu.setLieblingsFutter("Knochen");
		assertEquals("Knochen", neu.getLieblingsFutter(), errReturnValue + "getLieblingsFutter");

		assertThrows(NullPointerException.class, () -> {
			neu.setLieblingsFutter(null);
		}, errArgIllegal + "setLieblingsFutter");

		
		// --------------------------------------------------------------------------------
		neu.bewegeDich();
		neu.frissEtwas();
		neu.gibLaut();
		neu.streicheln();
	}

	@Test
	@Order(12)
	void testHund()
	{
		// --------------------------------------------------------------------------------
		// Test new mit Null und Leerstrings!!!!
		assertThrows(NullPointerException.class, () -> {
			new Hund(null, "Hund", "ID0815", 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Hund("", "Hund", "ID0815", 99);
		}, errNewEmptyStr);

		assertThrows(NullPointerException.class, () -> {
			new Hund("Waldi", null, "ID0815", 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Hund("Waldi", "", "ID0815", 99);
		}, errNewEmptyStr);

		assertThrows(NullPointerException.class, () -> {
			new Hund("Waldi", "Hund", null, 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Hund("Waldi", "Hund", "", 99);
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Hund("Waldi", "Hund", "ID0815", -1);
		}, errArgIllegal + "Alter");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Hund("Waldi", "Hund", "ID0815", 99);
		}, errNewNoError);
		
		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Hund neu = new Hund("Waldi", "Hund", "ID0815", 99);
		
		assertEquals("Waldi",  neu.getName(),   errReturnValue + "getName");
		assertEquals(99,       neu.getAlter(),  errReturnValue + "getAlter");
		assertEquals("ID0815", neu.getChipID(), errReturnValue + "getChipID");
		assertEquals("Hund",   neu.getRasse(),  errReturnValue + "getRasse");

		
		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			neu.setName("Test");
		}, errArgNoError);
		assertEquals("Test", neu.getName(), errReturnValue + "getName");

		assertThrows(NullPointerException.class, () -> {
			neu.setName(null);
		}, errArgIllegal + "setName");
		assertThrows(IllegalArgumentException.class, () -> {
			neu.setName("");
		}, errArgIllegal + "setName");

		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			neu.setAlter(3);
		}, errArgNoError);
		assertEquals(3, neu.getAlter(), errReturnValue + "getAlter");

		assertThrows(IllegalArgumentException.class, () -> {
			neu.setAlter(-1);
		}, errArgIllegal + "setAlter");

		// --------------------------------------------------------------------------------
		neu.setLieblingsFutter("Knochen");
		assertEquals("Knochen", neu.getLieblingsFutter(), errReturnValue + "getLieblingsFutter");

		assertThrows(NullPointerException.class, () -> {
			neu.setLieblingsFutter(null);
		}, errArgIllegal + "setLieblingsFutter");

		
		// --------------------------------------------------------------------------------
		neu.bewegeDich();
		neu.frissEtwas();
		neu.gibLaut();
		neu.geheGassi();
	}
}
