package ue211_Fuhrpark;
import static org.junit.jupiter.api.Assertions.*;

import java.util.ArrayList;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Order;

/***************************************************************************************************
 * 
 * JUnit Test für Fuhrpark.
 *
 * @author Prof. Gaisberger
 * @version 1.0
 *
 ***************************************************************************************************
 */

@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
//---------------------------------------------------------------------------------------------------
class _JUT_Fuhrpark
{
//	private static final String errNewNoError = "NEW: Mit diesen Parametern darf keine Exception ausgelöst werden!";
//	private static final String errNewNullPointer  = "NEW: Parameter darf nicht NULL sein (EXCEPTION fehlt): ";;
//	private static final String errNewEmptyStr = "NEW: Parameter darf kein Leerstring sein (EXCEPTION fehlt): ";
//	private static final String errArgIllegal = "Ungültiger Wert! (EXCEPTION fehlt): ";
//	private static final String errArgNoError = "Dieser Wert ist gültig! Hier darf KEINE Exception ausgelöst werden: ";
	private static final String errReturnValue = "Der zurückgegeben Wert ist ungültig! Methode: ";
	private static final String errReturnListe = "Die zurückgegebene Liste ist noch nicht korrekt! Methode: ";

	// -----------------------------------------------------------------
	// Variablen für Testumgebung
	private Fuhrpark  testPark = new Fuhrpark("JUnit-Test");
	Auto          aut1;
	Auto          aut2;
	LKW           lkw1;
	LKW           lkw2;
	Motorrad      mot1;
	Cabrio        cab1;
	Cabrio        cab2;
	Moped         mop1;

	// -----------------------------------------------------------------
	// Wird vor jedem Testlauf aufgerufen, damit eine entsprechende
	// Testumgebung initialisiert wird
	@BeforeEach
	void setUp() throws Exception
	{
		testPark = new Fuhrpark("JUnit-Test");

		aut1     = new Auto("AUT01", "Auto", 'D', 'C');
		aut2     = new Auto("AUT02", "Auto", 'S', 'K');
		lkw1     = new LKW("LKW01", "LKW", 'D', 20000);
		lkw2     = new LKW("LKW02", "LKW", 'S', 15000);
		mot1     = new Motorrad("MOT01", "Motorrad", 'S', 175);
		cab1     = new Cabrio("CAB01", "Cabrio", 'S', 'K');
		cab2     = new Cabrio("CAB02", "Cabrio", 'D', 'C');
		mop1     = new Moped("MOP01", "Moped", 'S');

		testPark.fahrzeugEinstellen(aut1);
		testPark.fahrzeugEinstellen(lkw1);
		testPark.fahrzeugEinstellen(mot1);
		testPark.fahrzeugEinstellen(cab1);
		testPark.fahrzeugEinstellen(aut2);
		testPark.fahrzeugEinstellen(lkw2);
		testPark.fahrzeugEinstellen(mop1);
		testPark.fahrzeugEinstellen(cab2);
	}
	// -----------------------------------------------------------------
	@Test
	@Order(21)
	void testIstFahrzeugEingestellt()
	{
		assertTrue(testPark.istFahrzeugEingestellt(aut1), errReturnValue + "istFahrzeugEingestellt");
		assertTrue(testPark.istFahrzeugEingestellt(mot1), errReturnValue + "istFahrzeugEingestellt");

		Auto neu = new Auto("FAZ999", "Auto", 'S','C');
		assertFalse(testPark.istFahrzeugEingestellt(neu), errReturnValue + "istFahrzeugEingestellt");

		testPark.fahrzeugEinstellen(neu);
		assertTrue(testPark.istFahrzeugEingestellt(neu), errReturnValue + "fahrzeugEinstellen");
	}

	@Test
	@Order(22)
	void testFahrzeugEntfernen()
	{
		assertTrue(testPark.istFahrzeugEingestellt(cab1), errReturnValue + "istFahrzeugEingestellt");
		testPark.fahrzeugEntfernen(cab1);
		assertFalse(testPark.istFahrzeugEingestellt(cab1), errReturnValue + "istFahrzeugEingestellt");
	}

	@Test
	void testFahrzeugListe()
	{
		testPark.printFahrzeugListe();
	}

	@Test
	void testPrintAutoListe()
	{
		testPark.printAutoListe();
	}

	@Test
	void testGetAnzAutos()
	{
		assertEquals(4,testPark.getAnzAutos(), errReturnValue + "getAnzAutos");
	}
	@Test
	void testGetAnzCabrios()
	{
		assertEquals(2,testPark.getAnzCabrios(), errReturnValue + "getAnzCabrios");
	}
	@Test
	void testGetAnzLKWs()
	{
		assertEquals(2,testPark.getAnzLKWs(), errReturnValue + "getAnzLKWs");
	}
	@Test
	void testGetAnzMotorrad()
	{
		assertEquals(1,testPark.getAnzMotorrad(), errReturnValue + "getAnzMotorrad");
	}
	@Test
	void testGetAnzMopedUndMotorrad()
	{
		assertEquals(2,testPark.getAnzMopedUndMotorrad(), errReturnValue + "getAnzMopedUndMotorrad");
	}



	@Test
	void testGetAutos()
	{
		// ----------------------------------------------
		List<Fahrzeug> v = new ArrayList<>();
		v.add(aut1);
		v.add(cab1);
		v.add(aut2);
		v.add(cab2);
		assertEquals(v, testPark.getAutos(), errReturnListe + "getAutos");
	}

	@Test
	void testGetCabrios()
	{
		// ----------------------------------------------
		List<Fahrzeug> v = new ArrayList<>();
		v.add(cab1);
		v.add(cab2);
		assertEquals(v, testPark.getCabrios(), errReturnListe + "getCabrios");
	}

	@Test
	void testGetLKWs()
	{
		// ----------------------------------------------
		List<Fahrzeug> v = new ArrayList<>();
		v.add(lkw1);
		v.add(lkw2);
		assertEquals(v, testPark.getLKWs(), errReturnListe + "getLKWs");
	}

	@Test
	void testGetMotorraeder()
	{
		// ----------------------------------------------
		List<Fahrzeug> v = new ArrayList<>();
		v.add(mot1);
		assertEquals(v, testPark.getMotorraeder(), errReturnListe + "getMotorraeder");
	}

	@Test
	void testGetAutosUndCabrios()
	{
		// ----------------------------------------------
		List<Fahrzeug> v = new ArrayList<>();
		v.add(aut1);
		v.add(cab2);
		v.add(cab2);
		assertEquals(v, testPark.getAutosUndCabrios('C'), errReturnListe + "getAutosUndCabrios");
	}

	@Test
	void testGetReiherUndPapageien()
	{
		// ----------------------------------------------
		assertEquals(mot1, testPark.getFahrzeugMitFIN("MOT01"), errReturnValue + "getFahrzeugMitFIN");
		assertEquals(null, testPark.getFahrzeugMitFIN("MOT1"),  errReturnValue + "getFahrzeugMitFIN");
	}
}
