package ue371_IP4Adresse;
import static org.junit.jupiter.api.Assertions.*;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;


/****************************************************************************************************
 * 
 * @author Prof. Gaisberger
 * @version 1.0
 *
 ****************************************************************************************************
 */

@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
//---------------------------------------------------------------------------------------------------
class _JUT_IP4Adresse_RegEx
{
	private static final String errNewNoError = "NEW: Mit diesen Parametern darf keine Exception ausgelöst werden!";
	private static final String errNewNullPointer  = "NEW: Parameter darf nicht NULL sein (EXCEPTION fehlt): ";;
//	private static final String errNewEmptyStr = "NEW: Parameter darf kein Leerstring sein (EXCEPTION fehlt): ";
	private static final String errArgIllegal = "Ungültiger Wert! (EXCEPTION fehlt): ";
//	private static final String errArgNoError = "Dieser Wert ist gültig! Hier darf KEINE Exception ausgelöst werden: ";
	private static final String errReturnValue = "Der zurückgegeben Wert ist ungültig! Methode: ";
//	private static final String errReturnListe = "Die zurückgegebene Liste ist noch nicht korrekt! Methode: ";

	// -------------------------------------------------------------------------------------
	@BeforeEach
	void setUp() throws Exception
	{
	  // Hier nicht benötigt
	}
	
	// -------------------------------------------------------------------------------------
	@Test
	@Order(10)
	void testIP4Adresse()
	{
		// ------------------------------------------------------------------------------
		// Teste Konstruktor
		assertThrows(NullPointerException.class, () -> {
			new IP4Adresse_RegEx(null);
		}, errNewNullPointer);

		assertThrows(IllegalArgumentException.class, () -> {
			new IP4Adresse_RegEx("Keine IP4Adresse");
		}, errArgIllegal + "IP4Adresse");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new IP4Adresse_RegEx("10.0.0.5");
		}, errNewNoError);
	}
	

	// -------------------------------------------------------------------------------------
	@Test
	@Order(20)
	void testGetterUndSetter()
	{
		IP4Adresse_RegEx fin = new IP4Adresse_RegEx("10.0.0.5");

		assertEquals("010.000.000.005", fin.getIPAdresse(),   errReturnValue + "getIPAdresse");
		assertEquals("00001010.00000000.00000000.00000101", fin.getIPAdresseBinary(),   errReturnValue + "getIPAdresseBinary");
	}
	
	
	// -------------------------------------------------------------------------------------
	@Test
	@Order(30)
	void testIsValid()
	{
		assertThrows(NullPointerException.class, () -> {
			IP4Adresse_RegEx.isValid(null);
		},   errArgIllegal + "Darf nicht null sein!");

		assertTrue(IP4Adresse_RegEx.isValid("192.168.0.1"),      errReturnValue + "isValid");
		assertTrue(IP4Adresse_RegEx.isValid("254.254.254.254"),  errReturnValue + "isValid");
		assertTrue(IP4Adresse_RegEx.isValid("1.1.1.1"), errReturnValue + "isValid");
		
		assertFalse(IP4Adresse_RegEx.isValid("1.0.1"),  errReturnValue + "isValid");
		assertFalse(IP4Adresse_RegEx.isValid("1.2.3.4.5"),  errReturnValue + "isValid");
	}

	
	// -------------------------------------------------------------------------------------
	@Test
	@Order(31)
	void testIsValidGenauer()
	{
		// ------------------------------------------------------------------------------
		// Vereinfachter Test! Gehört geschachtelt!
		// Annahme: 0 als Hostadresse ist zulässig.
		for (int   s=-10; s<300; s++) {
			
			String ip1 = "10.10.10." + s;
			String ip2 = "10.10." + s + ".10";
			String ip3 = "10." + s + ".10.10";
			String ip4 = s + ".10.10.10";
			
			if (s >= 0 && s <= 255) 
			{
				assertTrue(IP4Adresse_RegEx.isValid(ip1), errReturnValue + "isValid: " + ip1);
				assertTrue(IP4Adresse_RegEx.isValid(ip2), errReturnValue + "isValid: " + ip2);
				assertTrue(IP4Adresse_RegEx.isValid(ip3), errReturnValue + "isValid: " + ip3);
				assertTrue(IP4Adresse_RegEx.isValid(ip4), errReturnValue + "isValid: " + ip4);
			}
			else {
				assertFalse(IP4Adresse_RegEx.isValid(ip1), errReturnValue + "isValid: " + ip1);							
				assertFalse(IP4Adresse_RegEx.isValid(ip2), errReturnValue + "isValid: " + ip2);							
				assertFalse(IP4Adresse_RegEx.isValid(ip3), errReturnValue + "isValid: " + ip3);							
				assertFalse(IP4Adresse_RegEx.isValid(ip4), errReturnValue + "isValid: " + ip4);							
			}
		}
	}
	
	
}
