package ue640_Fraktal;

import java.awt.Graphics;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JPanel;


/****************************************************************************************************
 * 
 * TODO
 *
 * @author Prof. Gaisberger
 * @version 1.0
 *
 ****************************************************************************************************
 */

//---------------------------------------------------------------------------------------------------
@SuppressWarnings("serial")
public class Fraktal  extends  JPanel
{
	static final int itermax        = 2000;        // Maximale Anzahl von Iterationen
	static final int schwellenwert  = 35;          // bis zum Erreichen des Schwellwerts.

	
	
	
	// -------------------------------------------------------------------------------------
	// Bildbuffer
	private BufferedImage   bild;
	private int width = 300;
	private int heigth= 300;


	// -------------------------------------------------------------------------------------
	// Komplexe Ebene
	private double   		range  = 0.000005728;  
//	double   		range  = 0.000006728;  
	private Point2D.Double  origin = new Point2D.Double(-0.743645949, 0.131824149);
	private Point2D.Double  offset = new Point2D.Double(range/2, range/2);

	

	
	
	// -------------------------------------------------------------------------------------
	// KONSTRUKTOR
	public Fraktal(int width, int heigth)
	{
		setSize(width, heigth);
		
		bild = new BufferedImage(width, heigth, BufferedImage.TYPE_INT_RGB);

		setRange(0.000006728);
	}

	
	// -------------------------------------------------------------------------------------
	// INSTANZMETHODEN
	public void paint(Graphics g)
	{
		g.drawImage(bild, 0, 0, this);
	}
	
	/**
	 * Berechnet das Bild erneut und muss nach jeder Aktion erneut aufgerufen werden.
	 *
	 */
	public void berechneBild()
	{
		Point2D.Double topLeft = new Point2D.Double( origin.x - offset.x, origin.y - offset.y );
		
		ColorMap colors = new ColorMap();
		
		double scale = range / width;
		
		Graphics gBild = bild.createGraphics();

		for (int x = 0; x < width; x++)
		{
			for (int y = 0; y < heigth; y++)
			{
				Complex c = new Complex(topLeft.x + scale * x,
						                topLeft.y + scale * y);

				int iterationen = berechnePunkt(c);
				
				gBild.setColor(colors.map(iterationen));
				
				gBild.fillRect(x, y, 1, 1);
			}
			
			repaint();
		}
//		save("c:\\PortableRO\\imgx.jpg");
	}


	
	public int berechnePunkt(Complex c)
	{
		Complex z    = new Complex(0, 0);
		int     iterationen = 0;
		while ( (iterationen <= itermax) && (z.distanzUrsprung() < schwellenwert))
		{
			z = (z.mal(z)).plus(c);
			iterationen++;
		}
		return iterationen;	
	}

	/**
	 * Speichert das Bild in eine Datei.
	 *
	 * @param aFilename
	 */
	public void save(String aFilename)
	{
		if (aFilename == null)
			throw new IllegalArgumentException("argument to save() is null");

		String suffix = aFilename.substring(aFilename.lastIndexOf('.') + 1);
		if ("jpg".equalsIgnoreCase(suffix) || "png".equalsIgnoreCase(suffix))
		{
			try
			{
				File file = new File(aFilename);
				ImageIO.write(bild, suffix, file);
			} catch (IOException e)
			{
				e.printStackTrace();
			}
		}
		else
		{
			System.out.println("Error: filename must end in .jpg or .png");
		}
	}

	
	
	// -------------------------------------------------------------------------------------
	// GETTER / SETTER 
	@Override
	public void setSize(int width, int heigth)
	{
		if (width<20) width = 20;
		if (heigth<20) heigth = 20;
		
		this.width  = width;
		this.heigth = heigth;
		bild = new BufferedImage(width, heigth, BufferedImage.TYPE_INT_RGB);

		super.setSize(width, heigth);
	}
	
	/** 
	 * Setzt den Ursprung des Bildes im komplexen Raum.
	 *
	 * @param re
	 * @param img
	 */
	public void setOrigin(double re, double img)
	{
		origin = new Point2D.Double(re, img);
		System.out.printf("Origin: real: %.6f \t img: %.6f \n", re, img);
		
		berechneBild();
	}

	/**
	 * Setzt den Ursprung des Bildes im komplexen Raum.
	 *
	 * @param x
	 * @param y
	 */
	public void setOrigin( int x, int y) {
		
		double scale = range / width;

		setOrigin(origin.x - offset.x + scale*x , 
				  origin.y - offset.y + scale*y);
	}
	
	public void setRange(double range)
	{
		this.range = range;
		this.offset = new Point2D.Double(this.range/2, this.range/2);
		System.out.printf("Range: %.12f \n", range);
		
		berechneBild();
	}
	

	public void zoomIn() {
		setRange(range - range*0.2);
	}
	public void zoomOut() {
		setRange(range + range*0.2);
	}
	
	public void moveLeft() {
		setOrigin(width/2-20, width/2);
	}
	public void moveRight() {
		setOrigin(width/2+20, width/2);
	}
	
	public void moveUp() {
		setOrigin(width/2, width/2+20);
	}
	public void moveDown() {
		setOrigin(width/2, width/2-20);
	}

	public void sizeDown() {
		setSize(width-20, heigth-20);
		berechneBild();
	}

	public void sizeUp() {
		setSize(width+20, heigth+20);
		berechneBild();
	}

	
	// -------- Ende der Klasse 
}
